#include <PCI.h>
#include <Drivers.h>
#include <OS.h>
#include <ByteOrder.h>
#include <KernelExport.h>
#include <malloc.h>
#include <stdio.h>
#include <k_compat.h>
#include "linux_defs.h"
#include "linux_compat.h"
#include "mempool.h"

struct sk_buff *dev_alloc_skb(uint32 size)
{
	struct sk_buff *result = chunk_pool_get();
	if (result == NULL) return NULL;
	result->len = size;
	result->data_len = 0;
	result->ip_summed = 0;
	result->data = (u8*)(result + sizeof(struct sk_buff));
	return result;
}

void dev_kfree_skb(struct sk_buff *buffer)
{
	chunk_pool_put(buffer);
}

struct sk_buff *skb_pull(struct skb_queue *queue)
{
	struct sk_buff *the_buffer;
	struct skb_queue_item *the_item;
	if (queue->first == NULL) return NULL;
	the_item = queue->first;
	the_buffer = the_item->buff;
	queue->first = the_item->next;
	if (queue->first == NULL) queue->last = NULL;
	chunk_pool_put(the_item);
	return the_buffer;
}

void skb_offer(struct skb_queue *queue, struct sk_buff *the_buffer)
{
	struct skb_queue_item *the_item = (struct skb_queue_item *)chunk_pool_get();
	if (the_item == NULL)
	{
		// Paket wegwerfen
		dev_kfree_skb(the_buffer);
	 	return;
	}
	the_item->buff = the_buffer;
	the_item->next = NULL;
	if (queue->last == NULL)
	{
		queue->first = queue->last = the_item;
		return;
	}
	queue->last->next = the_item;
	queue->last = the_item;
}

int32 compat_timer_hook (timer *the_timer)
{
	timer_info *compat_timer = (timer_info *)the_timer;
	compat_timer->function(compat_timer->data);
	return 0;
}

void compat_add_timer(compat_timer_list *t)
{
	t->corresponds.data = t->data;
	t->corresponds.function = t->function;
	add_timer((timer*)&t->corresponds, compat_timer_hook, t->expires, B_ONE_SHOT_ABSOLUTE_TIMER);
}

void compat_del_timer(compat_timer_list *t)
{
	cancel_timer((timer*)&t->corresponds);
}

void get_random_bytes(unsigned char *buf, size_t len)
{
	while (len--)
		*buf++ = (unsigned char) random();
}

dma_addr_t pci_map_single(struct pci_dev *hwdev, void *ptr, size_t size, int direction)
{
	physical_entry area_phys_addr[2];
	get_memory_map(ptr, 1, &area_phys_addr[0], 1);
	return (dma_addr_t)(area_phys_addr[0].address);
}
