#ifndef _FORCEDETH_LINUX_COMPATH_
#define _FORCEDETH_LINUX_COMPATH_

#include "linux_defs.h"

// replacements for linux functions/definitions:
# define __iomem

#define le64_to_cpu(x)		B_LENDIAN_TO_HOST_INT64(((uint32)(x)))
#define cpu_to_le64(x)		B_HOST_TO_LENDIAN_INT64(((uint32)(x)))

struct net_device;

#define pci_dev net_device

struct sk_buff
{
	u8 *data;
	uint32 len;
	uint32 data_len;
	u8 ip_summed;
	struct net_device *dev;
};

struct sk_buff *dev_alloc_skb(uint32 size);
void dev_kfree_skb(struct sk_buff *buffer);
#define dev_kfree_skb_irq dev_kfree_skb

struct skb_queue_item
{
	struct skb_queue_item *next;
	struct sk_buff *buff;
};

struct skb_queue
{
	struct skb_queue_item *first;
	struct skb_queue_item *last;	
};

struct sk_buff *skb_pull(struct skb_queue *queue);
void skb_offer(struct skb_queue *queue, struct sk_buff *the_buffer);

#ifdef add_timer
#undef add_timer
#endif
extern status_t	add_timer(timer *t, timer_hook hook, bigtime_t period, int32 f);

#ifdef timer_list
#undef timer_list
#endif

typedef struct timer_info
{
	timer t;
	u_long data;
	void (*function)(u_long data);
} timer_info;

typedef struct compat_timer_list {
    struct compat_timer_list *next, *prev;
    bigtime_t expires;
    u_long data;
    void (*function)(u_long data);
    timer_info corresponds;
} compat_timer_list;
 

static inline void msleep(unsigned long msecs)
{
    do { unsigned long i; for (i=0;i<msecs;i++) snooze(1000); } while (0);
}

int32 compat_timer_hook (timer *the_timer);
void compat_add_timer(compat_timer_list *t);
void compat_del_timer(compat_timer_list *t);


static inline void mod_timer(compat_timer_list *t, bigtime_t newtime)
{
	compat_del_timer(t);
	t->expires = newtime;
	compat_add_timer(t);
}

void get_random_bytes(unsigned char *buf, size_t len);

static inline int is_valid_ether_addr( uint8 *addr )
{
	const char zaddr[6] = {0,};

	return !(addr[0]&1) && memcmp( addr, zaddr, 6);
}

#define wmb()		do { } while (0)
#define PCI_DMA_FROMDEVICE 0
#define PCI_DMA_TODEVICE 0
/* We use a memory pool and do not need to unmap memory:*/
#define pci_unmap_single(x...) do { } while (0)
/* mapping = finding out physical address: */
dma_addr_t pci_map_single(struct pci_dev *hwdev, void *ptr, size_t size, int direction);

#define __constant_htons htons

#define time_after(a,b)		((b) - (a) < 0)

#define	netif_start_queue(x...) do { } while (0)
/* returning 1 from nv_xmit... is enough */
#define netif_stop_queue(x...) do { } while (0)


#endif
